--
-- A level for Enigma
--
-- Copyright (c) 2003 Siegfried Fennig
-- Licensed under the GPL version 2.

levelw = 20
levelh = 97

create_world(levelw, levelh)
draw_border("st-greenbrown")
fill_floor("fl-leaves")

oxyd( 7, 7)
oxyd( 9, 15)
oxyd( 9, 25)
oxyd( 9, 82)
oxyd( 9, 88)
oxyd( 9, 63)
oxyd( 9, 35)
oxyd( 9, 45)
oxyd( 9, 55)
oxyd( 9, 72)
oxyd_shuffle()

draw_stones("st-greenbrown", {2,2},{1,0}, 16)
draw_stones("st-greenbrown", {4,4},{1,0}, 12)
draw_stones("st-greenbrown", {6,6},{1,0}, 8)
draw_stones("st-greenbrown", {2,94}, {1,0}, 16)
draw_stones("st-greenbrown", {4,92}, {1,0}, 12)
draw_stones("st-greenbrown", {6,90}, {1,0}, 8)
draw_stones("st-greenbrown", {2,3}, {0,1}, 91)
draw_stones("st-greenbrown", {4,5}, {0,1}, 87)
draw_stones("st-greenbrown", {6,7}, {0,1}, 83)
draw_stones("st-greenbrown", {17,3}, {0,1}, 91)
draw_stones("st-greenbrown", {15,5}, {0,1}, 87)
draw_stones("st-greenbrown", {13,7}, {0,1}, 83)
draw_stones("st-greenbrown", {7,8},{1,0}, 6)
draw_stones("st-greenbrown", {7,10},{1,0}, 6)
draw_stones("st-greenbrown", {14,10},{1,0}, 1)
draw_stones("st-greenbrown_hole", {7,11}, {1,0}, 1)
draw_stones("st-greenbrown", {7,12}, {0,1}, 2)
draw_stones("st-greenbrown", {9,11}, {0,1}, 4)
draw_stones("st-greenbrown", {10,12}, {0,1}, 3)
draw_stones("st-greenbrown", {11,12}, {0,1}, 4)
draw_stones("st-greenbrown", {8,15}, {0,1}, 3)
draw_stones("st-greenbrown", {9,16}, {0,1}, 2)
draw_stones("st-greenbrown", {10,17}, {0,1}, 1)
draw_stones("st-greenbrown", {12,17}, {0,1}, 2)
draw_stones("st-greenbrown", {8,19}, {1,0}, 3)
draw_stones("st-greenbrown", {8,20},{1,0}, 5)
draw_stones("st-greenbrown", {14,20},{1,0}, 1)
draw_stones("st-greenbrown", {8,22}, {0,1}, 7)
draw_stones("st-greenbrown", {10,22}, {0,1}, 7)
draw_stones("st-greenbrown", {11,22}, {0,1}, 7)
draw_stones("st-greenbrown", {9,26}, {0,1}, 3)
draw_stones("st-greenbrown", {7,30}, {0,1}, 6)
draw_stones("st-greenbrown", {8,30}, {0,1}, 6)
draw_stones("st-greenbrown", {9,30}, {0,1}, 5)
draw_stones("st-greenbrown", {10,30}, {1,0}, 3)
draw_stones("st-greenbrown", {11,32}, {0,1}, 7)
draw_stones("st-greenbrown", {7,37}, {1,0}, 4)
draw_stones("st-greenbrown", {9,38}, {1,0}, 2)
draw_stones("st-greenbrown", {7,39},{1,0}, 1)
draw_stones("st-greenbrown", {14,40}, {1,0}, 1)
draw_stones("st-greenbrown", {7,40}, {1,0}, 5)
draw_stones("st-greenbrown", {7,41}, {1,0}, 5)
draw_stones("st-greenbrown", {7,42}, {0,1}, 11)
draw_stones("st-greenbrown", {9,43}, {0,1}, 2)
draw_stones("st-greenbrown", {10,43}, {0,1}, 3)
draw_stones("st-greenbrown", {12,43}, {0,1}, 17)
draw_stones("st-greenbrown", {8,47}, {0,1}, 6)
draw_stones("st-greenbrown", {10,47}, {0,1}, 6)
draw_stones("st-greenbrown", {11,47}, {0,1}, 6)
draw_stones("st-greenbrown", {3,50}, {1,0}, 1)
draw_stones("st-greenbrown", {5,50}, {1,0}, 1)
draw_stones("st-greenbrown", {8,54},{1,0}, 3)
draw_stones("st-greenbrown", {10,55},{1,0}, 1)
draw_stones("st-greenbrown", {7,56}, {1,0}, 4)
draw_stones("st-greenbrown", {7,58}, {1,0}, 4)

draw_stones("st-greenbrown", {7,59}, {1,0}, 4)
draw_stones("st-greenbrown", {14,59}, {1,0}, 1)
draw_stones("st-greenbrown", {7,60}, {1,0}, 3)
draw_stones("st-greenbrown", {7,61}, {1,0}, 3)
draw_stones("st-greenbrown", {7,62}, {1,0}, 3)
draw_stones("st-greenbrown", {7,63}, {1,0}, 2)
draw_stones("st-greenbrown", {7,64}, {1,0}, 3)
draw_stones("st-greenbrown", {11,61}, {0,1}, 4)
draw_stones("st-greenbrown", {7,66}, {0,1}, 6)
draw_stones("st-greenbrown", {8,66}, {0,1}, 1)
draw_stones("st-greenbrown_hole", {8,67}, {0,1}, 5)
draw_stones("st-greenbrown", {9,66}, {0,1}, 6)
draw_stones("st-greenbrown", {10,66}, {0,1}, 7)
draw_stones("st-greenbrown", {12,66}, {0,1}, 7)
draw_stones("st-greenbrown", {5,68}, {1,0}, 1)
draw_stones("st-greenbrown", {14,68}, {1,0}, 1)
draw_stones("st-greenbrown", {7,73}, {0,1}, 6)
draw_stones("st-greenbrown", {9,74}, {1,0}, 4)
draw_stones("st-greenbrown", {9,76}, {1,0}, 4)
draw_stones("st-greenbrown", {9,77}, {1,0}, 4)
draw_stones("st-greenbrown", {5,77}, {1,0}, 1)
draw_stones("st-greenbrown", {9,78}, {1,0}, 4)
draw_stones("st-greenbrown", {7,80}, {1,0}, 5)
draw_stones("st-greenbrown", {7,82}, {1,0}, 1)
draw_stones("st-greenbrown", {10,82}, {0,1}, 4)
draw_stones("st-greenbrown", {12,82}, {0,1}, 4)
draw_stones("st-greenbrown", {8,84}, {0,1}, 2)
draw_stones("st-greenbrown", {7,86}, {1,0}, 6)
draw_stones("st-greenbrown", {14,86}, {1,0}, 1)
draw_stones("st-greenbrown", {7,88}, {1,0}, 1)
draw_stones("st-greenbrown", {10,88}, {1,0}, 3)

set_stone("st-death", 14,11)

set_item("it-hollow", 1,92)

set_stone("st-key_a", 5,40, {action="openclose", target="door1"})
set_stone("st-door_b", 11,8, {name="door1", type="h"})
set_item("it-key_a", 8,67)

set_stone( "st-key_b", 14,77, {action="openclose", target="door2"})
set_stone("st-door_b", 9,22, {name="door2", type="h"})
set_item("it-key_b", 12,7)

set_stone( "st-key_c", 5,59, {action="openclose", target="door3"})
set_stone("st-door_b", 8,55, {name="door3", type="v"})
set_item("it-key_c", 7,11)

set_stone("st-coinslot", 5,16, {action="openclose", target="door4"})
set_stone("st-door_b", 10,15, {name="door4", type="h"})
set_item("it-coin4", 5,12,{value=6})

oneway(15,5, enigma.EAST)
oneway(17,5, enigma.EAST)
oneway(4,7, enigma.EAST)
oneway(13,9, enigma.EAST)
oneway(5,10, enigma.SOUTH)
oneway(10,10, enigma.NORTH)
oneway(2,16, enigma.WEST)
oneway(6,15, enigma.EAST)
oneway(13,15, enigma.EAST)
oneway(10,18, enigma.EAST)
oneway(7,20, enigma.NORTH)
oneway(16,20, enigma.SOUTH)
oneway(15,23, enigma.EAST)
oneway(13,25, enigma.WEST)
oneway(6,28, enigma.WEST)
oneway(5,30, enigma.SOUTH)
oneway(11,30, enigma.NORTH)
oneway(14,30, enigma.NORTH)
oneway(17,30, enigma.EAST)
oneway(15,32, enigma.WEST)
oneway(6,38, enigma.EAST)
oneway(12,40, enigma.NORTH)
oneway(13,42, enigma.EAST)
oneway(2,45, enigma.WEST)
oneway(17,48, enigma.WEST)
oneway(9,50, enigma.NORTH)
oneway(14,50, enigma.SOUTH)
oneway(4,54, enigma.WEST)
oneway(2,56, enigma.EAST)
oneway(6,57, enigma.WEST)
oneway(15,58, enigma.EAST)
oneway(11,59, enigma.NORTH)
oneway(2,62, enigma.EAST)
oneway(6,65, enigma.EAST)
oneway(4,66, enigma.EAST)
oneway(15,69, enigma.WEST)
oneway(4,70, enigma.WEST)
oneway(6,72, enigma.WEST)
oneway(11,72, enigma.NORTH)
oneway(17,73, enigma.WEST)
oneway(13,75, enigma.WEST)
oneway(8,77, enigma.SOUTH)
oneway(6,79, enigma.WEST)
oneway(2,80, enigma.WEST)
oneway(13,80, enigma.EAST)
oneway(15,85, enigma.EAST)
oneway(5,86, enigma.SOUTH)
oneway(9,86, enigma.NORTH)
oneway(10,90, enigma.NORTH)
oneway(12,92, enigma.SOUTH)
oneway(8,94, enigma.NORTH)

set_actor("ac-blackball", 1.5, 92.5)
